@@ -230,10 +230,8 @@ MakefileGenerator
         build_proj->setExtraVars(basevars);
         build_proj->setExtraConfigs(basecfgs);
 
-        build_proj->read(project->projectFile());
-
-        //done
-        return createMakefileGenerator(build_proj);
+        if (build_proj->read(project->projectFile()))
+            return createMakefileGenerator(build_proj);
     }
     return 0;
 }
