@@ -42,6 +42,7 @@ import android.content.pm.PackageManager;
 import android.content.pm.PackageManager.NameNotFoundException;
 import android.content.res.Configuration;
 import android.graphics.drawable.ColorDrawable;
+import android.graphics.Rect;
 import android.net.LocalServerSocket;
 import android.net.LocalSocket;
 import android.os.Build;
@@ -59,12 +60,14 @@ import android.view.KeyCharacterMap;
 import android.view.KeyEvent;
 import android.view.Menu;
 import android.view.MenuItem;
+import android.view.MotionEvent;
 import android.view.Surface;
 import android.view.View;
 import android.view.ViewConfiguration;
 import android.view.ViewGroup;
 import android.view.WindowManager;
 import android.view.inputmethod.InputMethodManager;
+import android.view.ViewTreeObserver;
 
 import java.io.BufferedReader;
 import java.io.DataOutputStream;
@@ -76,7 +79,6 @@ import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.HashMap;
-import java.util.Iterator;
 
 public class QtActivityDelegate
 {
@@ -89,6 +91,8 @@ public class QtActivityDelegate
     private Method m_super_onKeyUp = null;
     private Method m_super_onConfigurationChanged = null;
     private Method m_super_onActivityResult = null;
+    private Method m_super_dispatchGenericMotionEvent = null;
+    private Method m_super_onWindowFocusChanged = null;
 
     private static final String NATIVE_LIBRARIES_KEY = "native.libraries";
     private static final String BUNDLED_LIBRARIES_KEY = "bundled.libraries";
@@ -122,6 +126,9 @@ public class QtActivityDelegate
     private boolean m_keyboardIsVisible = false;
     public boolean m_backKeyPressedSent = false;
     private long m_showHideTimeStamp = System.nanoTime();
+    private int m_portraitKeyboardHeight = 0;
+    private int m_landscapeKeyboardHeight = 0;
+    private int m_probeKeyboardHeightDelay = 50; // ms
 
     public void setFullScreen(boolean enterFullScreen)
     {
@@ -131,37 +138,28 @@ public class QtActivityDelegate
         if (m_fullScreen = enterFullScreen) {
             m_activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
             m_activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
-            if (Build.VERSION.SDK_INT >= 19) {
-                try {
-                    int ui_flag_immersive_sticky = View.class.getDeclaredField("SYSTEM_UI_FLAG_IMMERSIVE_STICKY").getInt(null);
-                    int ui_flag_layout_stable = View.class.getDeclaredField("SYSTEM_UI_FLAG_LAYOUT_STABLE").getInt(null);
-                    int ui_flag_layout_hide_navigation = View.class.getDeclaredField("SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION").getInt(null);
-                    int ui_flag_layout_fullscreen = View.class.getDeclaredField("SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN").getInt(null);
-                    int ui_flag_hide_navigation = View.class.getDeclaredField("SYSTEM_UI_FLAG_HIDE_NAVIGATION").getInt(null);
-                    int ui_flag_fullscreen = View.class.getDeclaredField("SYSTEM_UI_FLAG_FULLSCREEN").getInt(null);
-
+            try {
+                if (Build.VERSION.SDK_INT >= 19) {
+                    int flags = View.class.getDeclaredField("SYSTEM_UI_FLAG_HIDE_NAVIGATION").getInt(null);
+                    flags |= View.class.getDeclaredField("SYSTEM_UI_FLAG_LAYOUT_STABLE").getInt(null);
+                    flags |= View.class.getDeclaredField("SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION").getInt(null);
+                    flags |= View.class.getDeclaredField("SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN").getInt(null);
+                    flags |= View.class.getDeclaredField("SYSTEM_UI_FLAG_FULLSCREEN").getInt(null);
+                    flags |= View.class.getDeclaredField("SYSTEM_UI_FLAG_IMMERSIVE_STICKY").getInt(null);
                     Method m = View.class.getMethod("setSystemUiVisibility", int.class);
-                    m.invoke(m_activity.getWindow().getDecorView(),
-                             ui_flag_layout_stable
-                             | ui_flag_layout_hide_navigation
-                             | ui_flag_layout_fullscreen
-                             | ui_flag_hide_navigation
-                             | ui_flag_fullscreen
-                             | ui_flag_immersive_sticky
-                             | View.INVISIBLE);
-                } catch (Exception e) {
-                    e.printStackTrace();
+                    m.invoke(m_activity.getWindow().getDecorView(), flags | View.INVISIBLE);
                 }
+            } catch (Exception e) {
+                e.printStackTrace();
             }
         } else {
             m_activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
             m_activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
-            if (Build.VERSION.SDK_INT >= 19) {
+            if (Build.VERSION.SDK_INT >= 14) {
                 try {
                     int ui_flag_visible = View.class.getDeclaredField("SYSTEM_UI_FLAG_VISIBLE").getInt(null);
                     Method m = View.class.getMethod("setSystemUiVisibility", int.class);
-                    m.invoke(m_activity.getWindow().getDecorView(),
-                             ui_flag_visible);
+                    m.invoke(m_activity.getWindow().getDecorView(), ui_flag_visible);
                 } catch (Exception e) {
                     e.printStackTrace();
                 }
@@ -203,11 +201,21 @@ public class QtActivityDelegate
     private final int ImhUrlCharactersOnly = 0x400000;
     private final int ImhLatinOnly = 0x800000;
 
+    // enter key type - must be kept in sync with QTDIR/src/corelib/global/qnamespace.h
+    private final int EnterKeyDefault = 0;
+    private final int EnterKeyReturn = 1;
+    private final int EnterKeyDone = 2;
+    private final int EnterKeyGo = 3;
+    private final int EnterKeySend = 4;
+    private final int EnterKeySearch = 5;
+    private final int EnterKeyNext = 6;
+    private final int EnterKeyPrevious = 7;
+
     // application state
-    private final int ApplicationSuspended = 0x0;
-    private final int ApplicationHidden = 0x1;
-    private final int ApplicationInactive = 0x2;
-    private final int ApplicationActive = 0x4;
+    public static final int ApplicationSuspended = 0x0;
+    public static final int ApplicationHidden = 0x1;
+    public static final int ApplicationInactive = 0x2;
+    public static final int ApplicationActive = 0x4;
 
 
     public boolean setKeyboardVisibility(boolean visibility, long timeStamp)
@@ -239,20 +247,57 @@ public class QtActivityDelegate
         }, 5);
     }
 
-    public void showSoftwareKeyboard(int x, int y, int width, int height, int inputHints)
+    public void showSoftwareKeyboard(final int x, final int y, final int width, final int height, final int inputHints, final int enterKeyType)
     {
         if (m_imm == null)
             return;
 
-        if (m_softInputMode == 0 && height > m_layout.getHeight() * 2 / 3)
-           m_activity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_UNCHANGED | WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE);
-        else if (m_softInputMode == 0)
-            m_activity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_UNCHANGED | WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN);
+        DisplayMetrics metrics = new DisplayMetrics();
+        m_activity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
+
+        // If the screen is in portrait mode than we estimate that keyboard height will not be higher than 2/5 of the screen.
+        // else than we estimate that keyboard height will not be higher than 2/3 of the screen
+        final int visibleHeight;
+        if (metrics.widthPixels < metrics.heightPixels)
+            visibleHeight = m_portraitKeyboardHeight != 0 ? m_portraitKeyboardHeight : metrics.heightPixels * 3 / 5;
         else
+            visibleHeight = m_landscapeKeyboardHeight != 0 ? m_landscapeKeyboardHeight : metrics.heightPixels / 3;
+
+        if (m_softInputMode != 0) {
             m_activity.getWindow().setSoftInputMode(m_softInputMode);
+        } else {
+            if (height > visibleHeight)
+                m_activity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_UNCHANGED | WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE);
+            else
+                m_activity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_UNCHANGED | WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN);
+        }
 
         int initialCapsMode = 0;
+
         int imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_DONE;
+
+        switch (enterKeyType) {
+        case EnterKeyReturn:
+            imeOptions = android.view.inputmethod.EditorInfo.IME_FLAG_NO_ENTER_ACTION;
+            break;
+        case EnterKeyGo:
+            imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_GO;
+            break;
+        case EnterKeySend:
+            imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_SEND;
+            break;
+        case EnterKeySearch:
+            imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_SEARCH;
+            break;
+        case EnterKeyNext:
+            imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_NEXT;
+            break;
+        case EnterKeyPrevious:
+            if (Build.VERSION.SDK_INT > 10)
+                imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_PREVIOUS;
+            break;
+        }
+
         int inputType = android.text.InputType.TYPE_CLASS_TEXT;
 
         if ((inputHints & (ImhPreferNumbers | ImhDigitsOnly | ImhFormattedNumbersOnly)) != 0) {
@@ -278,7 +323,9 @@ public class QtActivityDelegate
             if ((inputHints & (ImhEmailCharactersOnly | ImhUrlCharactersOnly)) != 0) {
                 if ((inputHints & ImhUrlCharactersOnly) != 0) {
                     inputType |= android.text.InputType.TYPE_TEXT_VARIATION_URI;
-                    imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_GO;
+
+                    if (enterKeyType == 0) // not explicitly overridden
+                        imeOptions = android.view.inputmethod.EditorInfo.IME_ACTION_GO;
                 } else if ((inputHints & ImhEmailCharactersOnly) != 0) {
                     inputType |= android.text.InputType.TYPE_TEXT_VARIATION_EMAIL_ADDRESS;
                 }
@@ -305,15 +352,14 @@ public class QtActivityDelegate
             }
         }
 
-        if ((inputHints & ImhMultiLine) != 0)
+        if (enterKeyType == 0 && (inputHints & ImhMultiLine) != 0)
             imeOptions = android.view.inputmethod.EditorInfo.IME_FLAG_NO_ENTER_ACTION;
 
         m_editText.setInitialCapsMode(initialCapsMode);
         m_editText.setImeOptions(imeOptions);
         m_editText.setInputType(inputType);
 
-        m_layout.removeView(m_editText);
-        m_layout.addView(m_editText, new QtLayout.LayoutParams(width, height, x, y));
+        m_layout.setLayoutParams(m_editText, new QtLayout.LayoutParams(width, height, x, y), false);
         m_editText.requestFocus();
         m_editText.postDelayed(new Runnable() {
             @Override
@@ -327,6 +373,38 @@ public class QtActivityDelegate
                                 //FALLTHROUGH
                             case InputMethodManager.RESULT_UNCHANGED_SHOWN:
                                 setKeyboardVisibility(true, System.nanoTime());
+                                if (m_softInputMode == 0) {
+                                    // probe for real keyboard height
+                                    m_layout.postDelayed(new Runnable() {
+                                            @Override
+                                            public void run() {
+                                                if (!m_keyboardIsVisible)
+                                                    return;
+                                                DisplayMetrics metrics = new DisplayMetrics();
+                                                m_activity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
+                                                Rect r = new Rect();
+                                                m_activity.getWindow().getDecorView().getWindowVisibleDisplayFrame(r);
+                                                if (metrics.heightPixels != r.bottom) {
+                                                    if (metrics.widthPixels > metrics.heightPixels) { // landscape
+                                                        if (m_landscapeKeyboardHeight != r.bottom) {
+                                                            m_landscapeKeyboardHeight = r.bottom;
+                                                            showSoftwareKeyboard(x, y, width, height, inputHints, enterKeyType);
+                                                        }
+                                                    } else {
+                                                        if (m_portraitKeyboardHeight != r.bottom) {
+                                                            m_portraitKeyboardHeight = r.bottom;
+                                                            showSoftwareKeyboard(x, y, width, height, inputHints, enterKeyType);
+                                                        }
+                                                    }
+                                                } else {
+                                                    // no luck ?
+                                                    // maybe the delay was too short, so let's make it longer
+                                                    if (m_probeKeyboardHeightDelay < 1000)
+                                                        m_probeKeyboardHeightDelay *= 2;
+                                                }
+                                            }
+                                        }, m_probeKeyboardHeightDelay);
+                                    }
                                 break;
                             case InputMethodManager.RESULT_HIDDEN:
                             case InputMethodManager.RESULT_UNCHANGED_HIDDEN:
@@ -439,6 +517,14 @@ public class QtActivityDelegate
             m_super_onKeyUp = m_activity.getClass().getMethod("super_onKeyUp", Integer.TYPE, KeyEvent.class);
             m_super_onConfigurationChanged = m_activity.getClass().getMethod("super_onConfigurationChanged", Configuration.class);
             m_super_onActivityResult = m_activity.getClass().getMethod("super_onActivityResult", Integer.TYPE, Integer.TYPE, Intent.class);
+            m_super_onWindowFocusChanged = m_activity.getClass().getMethod("super_onWindowFocusChanged", Boolean.TYPE);
+            if (Build.VERSION.SDK_INT >= 12) {
+                try {
+                    m_super_dispatchGenericMotionEvent = m_activity.getClass().getMethod("super_dispatchGenericMotionEvent", MotionEvent.class);
+                } catch (Exception e) {
+                }
+            }
+
         } catch (Exception e) {
             e.printStackTrace();
             return false;
@@ -815,6 +901,25 @@ public class QtActivityDelegate
 
         QtNative.handleOrientationChanged(rotation, m_nativeOrientation);
         m_currentRotation = rotation;
+
+        m_layout.getViewTreeObserver().addOnPreDrawListener(new ViewTreeObserver.OnPreDrawListener() {
+            @Override
+            public boolean onPreDraw() {
+                if (!m_keyboardIsVisible)
+                    return true;
+
+                Rect r = new Rect();
+                m_activity.getWindow().getDecorView().getWindowVisibleDisplayFrame(r);
+                DisplayMetrics metrics = new DisplayMetrics();
+                m_activity.getWindowManager().getDefaultDisplay().getMetrics(metrics);
+                final int kbHeight = metrics.heightPixels - r.bottom;
+                final int[] location = new int[2];
+                m_layout.getLocationOnScreen(location);
+                QtNative.keyboardGeometryChanged(location[0], r.bottom - location[1],
+                                                 r.width(), kbHeight);
+                return true;
+            }
+        });
     }
 
     public void initializeAccessibility()
@@ -836,6 +941,16 @@ public class QtActivityDelegate
         }
     }
 
+    public void onWindowFocusChanged(boolean hasFocus) {
+        try {
+            m_super_onWindowFocusChanged.invoke(m_activity, hasFocus);
+        } catch (Exception e) {
+            e.printStackTrace();
+        }
+        if (hasFocus)
+            updateFullScreen();
+    }
+
     public void onConfigurationChanged(Configuration configuration)
     {
         try {
@@ -854,6 +969,7 @@ public class QtActivityDelegate
     public void onDestroy()
     {
         if (m_quitApp) {
+            QtNative.terminateQt();
             if (m_debuggerProcess != null)
                 m_debuggerProcess.destroy();
             System.exit(0);// FIXME remove it or find a better way
@@ -862,24 +978,15 @@ public class QtActivityDelegate
 
     public void onPause()
     {
-        QtNative.updateApplicationState(ApplicationInactive);
+        QtNative.setApplicationState(ApplicationInactive);
     }
 
     public void onResume()
     {
-        // fire all lostActions
-        synchronized (QtNative.m_mainActivityMutex)
-        {
-            Iterator<Runnable> itr = QtNative.getLostActions().iterator();
-            while (itr.hasNext())
-                m_activity.runOnUiThread(itr.next());
-
-            QtNative.updateApplicationState(ApplicationActive);
-            if (m_started) {
-                QtNative.clearLostActions();
-                QtNative.updateWindow();
-                updateFullScreen(); // Suspending the app clears the immersive mode, so we need to set it again.
-            }
+        QtNative.setApplicationState(ApplicationActive);
+        if (m_started) {
+            QtNative.updateWindow();
+            updateFullScreen(); // Suspending the app clears the immersive mode, so we need to set it again.
         }
     }
 
@@ -902,7 +1009,7 @@ public class QtActivityDelegate
 
     public void onStop()
     {
-        QtNative.updateApplicationState(ApplicationSuspended);
+        QtNative.setApplicationState(ApplicationSuspended);
     }
 
     public Object onRetainNonConfigurationInstance()
@@ -1007,6 +1114,9 @@ public class QtActivityDelegate
             QtNative.keyUp(0, event.getCharacters().charAt(0), event.getMetaState(), event.getRepeatCount() > 0);
         }
 
+        if (QtNative.dispatchKeyEvent(event))
+            return true;
+
         try {
             return (Boolean) m_super_dispatchKeyEvent.invoke(m_activity, event);
         } catch (Exception e) {
@@ -1091,12 +1201,10 @@ public class QtActivityDelegate
                     if (Build.VERSION.SDK_INT < 11 || w <= 0 || h <= 0) {
                         m_activity.openContextMenu(m_layout);
                     } else if (Build.VERSION.SDK_INT < 14) {
-                        m_layout.removeView(m_editText);
-                        m_layout.addView(m_editText, new QtLayout.LayoutParams(w, h, x, y));
+                        m_layout.setLayoutParams(m_editText, new QtLayout.LayoutParams(w, h, x, y), false);
                         QtPopupMenu.getInstance().showMenu(m_editText);
                     } else {
-                        m_layout.removeView(m_editText);
-                        m_layout.addView(m_editText, new QtLayout.LayoutParams(w, h, x, y));
+                        m_layout.setLayoutParams(m_editText, new QtLayout.LayoutParams(w, h, x, y), false);
                         QtPopupMenu14.getInstance().showMenu(m_editText);
                     }
                 }
@@ -1275,4 +1383,17 @@ public class QtActivityDelegate
             m_layout.moveChild(view, index);
         }
     }
+
+    public boolean dispatchGenericMotionEvent (MotionEvent ev)
+    {
+        if (m_started && QtNative.dispatchGenericMotionEvent(ev))
+            return true;
+
+        try {
+            return (Boolean) m_super_dispatchGenericMotionEvent.invoke(m_activity, ev);
+        } catch (Exception e) {
+            e.printStackTrace();
+        }
+        return false;
+    }
 }
