@@ -1,7 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2013 John Layt <jlayt@kde.org>
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -56,10 +56,12 @@ static QTimeZonePrivate *newBackendTimeZone()
 #else
 #if defined Q_OS_MAC
     return new QMacTimeZonePrivate();
+#elif defined Q_OS_ANDROID
+    return new QAndroidTimeZonePrivate();
 #elif defined Q_OS_UNIX
     return new QTzTimeZonePrivate();
     // Registry based timezone backend not available on WinRT
-#elif defined Q_OS_WIN && !defined Q_OS_WINRT
+#elif defined Q_OS_WIN
     return new QWinTimeZonePrivate();
 #elif defined QT_USE_ICU
     return new QIcuTimeZonePrivate();
@@ -81,10 +83,12 @@ static QTimeZonePrivate *newBackendTimeZone(const QByteArray &ianaId)
 #else
 #if defined Q_OS_MAC
     return new QMacTimeZonePrivate(ianaId);
+#elif defined Q_OS_ANDROID
+    return new QAndroidTimeZonePrivate(ianaId);
 #elif defined Q_OS_UNIX
     return new QTzTimeZonePrivate(ianaId);
     // Registry based timezone backend not available on WinRT
-#elif defined Q_OS_WIN && !defined Q_OS_WINRT
+#elif defined Q_OS_WIN
     return new QWinTimeZonePrivate(ianaId);
 #elif defined QT_USE_ICU
     return new QIcuTimeZonePrivate(ianaId);
@@ -112,8 +116,9 @@ Q_GLOBAL_STATIC(QTimeZoneSingleton, global_tz);
     \class QTimeZone
     \inmodule QtCore
     \since 5.2
-    \brief The QTimeZone class converts between between UTC and local time in a
-           specific time zone.
+
+    \brief The QTimeZone class converts between UTC and local time in a specific
+           time zone.
 
     \threadsafe
 
@@ -194,14 +199,15 @@ Q_GLOBAL_STATIC(QTimeZoneSingleton, global_tz);
     The difference between UTC and the local time in a time zone is expressed
     as an offset in seconds from UTC, i.e. the number of seconds to add to UTC
     to obtain the local time.  The total offset is comprised of two component
-    parts, the standard time offset and the daylight time offset.  The standard
-    time offset is the number of seconds to add to UTC to obtain standard time
-    in the time zone.  The daylight time offset is the number of seconds to add
-    to the standard time offset to obtain daylight time in the time zone.
+    parts, the standard time offset and the daylight-saving time offset.  The
+    standard time offset is the number of seconds to add to UTC to obtain
+    standard time in the time zone.  The daylight-saving time offset is the
+    number of seconds to add to the standard time offset to obtain
+    daylight-saving time (abbreviated DST and sometimes called "daylight time"
+    or "summer time") in the time zone.
 
-    Note that the standard and daylight offsets for a time zone may change over
-    time as countries have changed daylight time laws or even their standard
-    time offset.
+    Note that the standard and DST offsets for a time zone may change over time
+    as countries have changed DST laws or even their standard time offset.
 
     \section2 License
 
@@ -236,21 +242,20 @@ Q_GLOBAL_STATIC(QTimeZoneSingleton, global_tz);
     \enum QTimeZone::TimeType
 
     The type of time zone time, for example when requesting the name.  In time
-    zones that do not apply daylight time, all three values may return the
-    same result.
+    zones that do not apply DST, all three values may return the same result.
 
     \value StandardTime
-           The standard time in a time zone, i.e. when Daylight Savings is not
+           The standard time in a time zone, i.e. when Daylight-Saving is not
            in effect.
            For example when formatting a display name this will show something
            like "Pacific Standard Time".
     \value DaylightTime
-           A time when Daylight Savings is in effect.
+           A time when Daylight-Saving is in effect.
            For example when formatting a display name this will show something
-           like "Pacific daylight time".
+           like "Pacific daylight-saving time".
     \value GenericTime
-           A time which is not specifically Standard or Daylight time, either
-           an unknown time or a neutral form.
+           A time which is not specifically Standard or Daylight-Saving time,
+           either an unknown time or a neutral form.
            For example when formatting a display name this will show something
            like "Pacific Time".
 */
@@ -281,11 +286,11 @@ Q_GLOBAL_STATIC(QTimeZoneSingleton, global_tz);
     \li OffsetData::atUtc  The datetime of the offset data in UTC time.
     \li OffsetData::offsetFromUtc  The total offset from UTC in effect at the datetime.
     \li OffsetData::standardTimeOffset  The standard time offset component of the total offset.
-    \li OffsetData::daylightTimeOffset  The daylight time offset component of the total offset.
+    \li OffsetData::daylightTimeOffset  The DST offset component of the total offset.
     \li OffsetData::abbreviation  The abbreviation in effect at the datetime.
     \endlist
 
-    For example, for time zone "Europe/Berlin" the OffsetDate in standard and daylight time might be:
+    For example, for time zone "Europe/Berlin" the OffsetDate in standard and DST might be:
 
     \list
     \li atUtc = QDateTime(QDate(2013, 1, 1), QTime(0, 0, 0), Qt::UTC)
@@ -306,9 +311,8 @@ Q_GLOBAL_STATIC(QTimeZoneSingleton, global_tz);
 
 /*!
     \typedef QTimeZone::OffsetDataList
-    \relates QTimeZone
 
-    Synonym for QList<OffsetData>.
+    Synonym for QVector<OffsetData>.
 */
 
 /*!
@@ -374,12 +378,10 @@ QTimeZone::QTimeZone(int offsetSeconds)
 
 QTimeZone::QTimeZone(const QByteArray &ianaId, int offsetSeconds, const QString &name,
                      const QString &abbreviation, QLocale::Country country, const QString &comment)
+    : d()
 {
-    // ianaId must be a valid ID and must not clash with the standard system names
-    if (QTimeZonePrivate::isValidId(ianaId) && !availableTimeZoneIds().contains(ianaId))
+    if (!isTimeZoneIdAvailable(ianaId))
         d = new QUtcTimeZonePrivate(ianaId, offsetSeconds, name, abbreviation, country, comment);
-    else
-        d = 0;
 }
 
 /*!
@@ -529,8 +531,7 @@ QString QTimeZone::comment() const
     If the \a locale is not provided then the application default locale will
     be used.
 
-    The display name may change depending on daylight time or historical
-    events.
+    The display name may change depending on DST or historical events.
 
     \sa abbreviation()
 */
@@ -570,8 +571,7 @@ QString QTimeZone::displayName(TimeType timeType, NameType nameType,
 
 /*!
     Returns the time zone abbreviation at the given \a atDateTime.  The
-    abbreviation may change depending on daylight time or even
-    historical events.
+    abbreviation may change depending on DST or even historical events.
 
     Note that the abbreviation is not guaranteed to be unique to this time zone
     and should not be used in place of the ID or display name.
@@ -590,13 +590,13 @@ QString QTimeZone::abbreviation(const QDateTime &atDateTime) const
 /*!
     Returns the total effective offset at the given \a atDateTime, i.e. the
     number of seconds to add to UTC to obtain the local time.  This includes
-    any daylight time offset that may be in effect, i.e. it is the sum of
+    any DST offset that may be in effect, i.e. it is the sum of
     standardTimeOffset() and daylightTimeOffset() for the given datetime.
 
     For example, for the time zone "Europe/Berlin" the standard time offset is
-    +3600 seconds and the daylight time offset is +3600 seconds.  During standard
-    time offsetFromUtc() will return +3600 (UTC+01:00), and during daylight time
-    it will return +7200 (UTC+02:00).
+    +3600 seconds and the DST offset is +3600 seconds.  During standard time
+    offsetFromUtc() will return +3600 (UTC+01:00), and during DST it will
+    return +7200 (UTC+02:00).
 
     \sa standardTimeOffset(), daylightTimeOffset()
 */
@@ -612,11 +612,11 @@ int QTimeZone::offsetFromUtc(const QDateTime &atDateTime) const
 /*!
     Returns the standard time offset at the given \a atDateTime, i.e. the
     number of seconds to add to UTC to obtain the local Standard Time.  This
-    excludes any daylight time offset that may be in effect.
+    excludes any DST offset that may be in effect.
 
     For example, for the time zone "Europe/Berlin" the standard time offset is
-    +3600 seconds.  During both standard and daylight time offsetFromUtc() will
-    return +3600 (UTC+01:00).
+    +3600 seconds.  During both standard and DST offsetFromUtc() will return
+    +3600 (UTC+01:00).
 
     \sa offsetFromUtc(), daylightTimeOffset()
 */
@@ -630,13 +630,13 @@ int QTimeZone::standardTimeOffset(const QDateTime &atDateTime) const
 }
 
 /*!
-    Returns the daylight time offset at the given \a atDateTime, i.e. the
-    number of seconds to add to the standard time offset to obtain the local
-    daylight time.
+    Returns the daylight-saving time offset at the given \a atDateTime,
+    i.e. the number of seconds to add to the standard time offset to obtain the
+    local daylight-saving time.
 
-    For example, for the time zone "Europe/Berlin" the daylight time offset
-    is +3600 seconds.  During standard time daylightTimeOffset() will return
-    0, and during daylight time it will return +3600.
+    For example, for the time zone "Europe/Berlin" the DST offset is +3600
+    seconds.  During standard time daylightTimeOffset() will return 0, and when
+    daylight-saving is in effect it will return +3600.
 
     \sa offsetFromUtc(), standardTimeOffset()
 */
@@ -650,7 +650,7 @@ int QTimeZone::daylightTimeOffset(const QDateTime &atDateTime) const
 }
 
 /*!
-    Returns \c true if the time zone has observed daylight time at any time.
+    Returns \c true if the time zone has practiced daylight-saving at any time.
 
     \sa isDaylightTime(), daylightTimeOffset()
 */
@@ -664,7 +664,7 @@ bool QTimeZone::hasDaylightTime() const
 }
 
 /*!
-    Returns \c true if the given \a atDateTime is in daylight time.
+    Returns \c true if daylight-saving was in effect at the given \a atDateTime.
 
     \sa hasDaylightTime(), daylightTimeOffset()
 */
@@ -695,6 +695,11 @@ QTimeZone::OffsetData QTimeZone::offsetData(const QDateTime &forDateTime) const
 
 /*!
     Returns \c true if the system backend supports obtaining transitions.
+
+    Transitions are changes in the time-zone: these happen when DST turns on or
+    off and when authorities alter the offsets for the time-zone.
+
+    \sa nextTransition(), previousTransition(), transitions()
 */
 
 bool QTimeZone::hasTransitions() const
@@ -762,6 +767,7 @@ QTimeZone::OffsetDataList QTimeZone::transitions(const QDateTime &fromDateTime,
     if (hasTransitions()) {
         QTimeZonePrivate::DataList plist = d->transitions(fromDateTime.toMSecsSinceEpoch(),
                                                           toDateTime.toMSecsSinceEpoch());
+        list.reserve(plist.count());
         foreach (const QTimeZonePrivate::Data &pdata, plist)
             list.append(d->toOffsetData(pdata));
     }
@@ -784,6 +790,29 @@ QByteArray QTimeZone::systemTimeZoneId()
     return global_tz->backend->systemTimeZoneId();
 }
 
+/*!
+    \since 5.5
+    Returns a QTimeZone object that refers to the local system time, as
+    specified by systemTimeZoneId().
+
+    \sa utc()
+*/
+QTimeZone QTimeZone::systemTimeZone()
+{
+    return QTimeZone(QTimeZone::systemTimeZoneId());
+}
+
+/*!
+    \since 5.5
+    Returns a QTimeZone object that refers to UTC (Universal Time Coordinated).
+
+    \sa systemTimeZone()
+*/
+QTimeZone QTimeZone::utc()
+{
+    return QTimeZone(QTimeZonePrivate::utcQByteArray());
+}
+
 /*!
     Returns \c true if a given time zone \a ianaId is available on this system.
 
@@ -794,7 +823,20 @@ bool QTimeZone::isTimeZoneIdAvailable(const QByteArray &ianaId)
 {
     // isValidId is not strictly required, but faster to weed out invalid
     // IDs as availableTimeZoneIds() may be slow
-    return (QTimeZonePrivate::isValidId(ianaId) && (availableTimeZoneIds().contains(ianaId)));
+    if (!QTimeZonePrivate::isValidId(ianaId))
+        return false;
+    const QList<QByteArray> tzIds = availableTimeZoneIds();
+    return std::binary_search(tzIds.begin(), tzIds.end(), ianaId);
+}
+
+static QList<QByteArray> set_union(const QList<QByteArray> &l1, const QList<QByteArray> &l2)
+{
+    QList<QByteArray> result;
+    result.reserve(l1.size() + l2.size());
+    std::set_union(l1.begin(), l1.end(),
+                   l2.begin(), l2.end(),
+                   std::back_inserter(result));
+    return result;
 }
 
 /*!
@@ -805,11 +847,8 @@ bool QTimeZone::isTimeZoneIdAvailable(const QByteArray &ianaId)
 
 QList<QByteArray> QTimeZone::availableTimeZoneIds()
 {
-    QSet<QByteArray> set = QUtcTimeZonePrivate().availableTimeZoneIds()
-                           + global_tz->backend->availableTimeZoneIds();
-    QList<QByteArray> list = set.toList();
-    std::sort(list.begin(), list.end());
-    return list;
+    return set_union(QUtcTimeZonePrivate().availableTimeZoneIds(),
+                     global_tz->backend->availableTimeZoneIds());
 }
 
 /*!
@@ -825,11 +864,8 @@ QList<QByteArray> QTimeZone::availableTimeZoneIds()
 
 QList<QByteArray> QTimeZone::availableTimeZoneIds(QLocale::Country country)
 {
-    QSet<QByteArray> set = QUtcTimeZonePrivate().availableTimeZoneIds(country)
-                           + global_tz->backend->availableTimeZoneIds(country);
-    QList<QByteArray> list = set.toList();
-    std::sort(list.begin(), list.end());
-    return list;
+    return set_union(QUtcTimeZonePrivate().availableTimeZoneIds(country),
+                     global_tz->backend->availableTimeZoneIds(country));
 }
 
 /*!
@@ -841,11 +877,8 @@ QList<QByteArray> QTimeZone::availableTimeZoneIds(QLocale::Country country)
 
 QList<QByteArray> QTimeZone::availableTimeZoneIds(int offsetSeconds)
 {
-    QSet<QByteArray> set = QUtcTimeZonePrivate().availableTimeZoneIds(offsetSeconds)
-                           + global_tz->backend->availableTimeZoneIds(offsetSeconds);
-    QList<QByteArray> list = set.toList();
-    std::sort(list.begin(), list.end());
-    return list;
+    return set_union(QUtcTimeZonePrivate().availableTimeZoneIds(offsetSeconds),
+                     global_tz->backend->availableTimeZoneIds(offsetSeconds));
 }
 
 /*!
@@ -953,9 +986,10 @@ QDataStream &operator>>(QDataStream &ds, QTimeZone &tz)
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug dbg, const QTimeZone &tz)
 {
+    QDebugStateSaver saver(dbg);
     //TODO Include backend and data version details?
     dbg.nospace() << "QTimeZone(" << QString::fromUtf8(tz.id()) << ')';
-    return dbg.space();
+    return dbg;
 }
 #endif
 
