@@ -31,11 +31,6 @@
 **
 ****************************************************************************/
 
-#if _WIN32_WINNT < 0x0500
-#undef _WIN32_WINNT
-#define _WIN32_WINNT 0x0500
-#endif
-
 #include "qwindowsintegration.h"
 #include "qwindowsfontengine.h"
 #include "qwindowsnativeimage.h"
@@ -103,17 +98,6 @@ static void resolveGetCharWidthI()
     ptrGetCharWidthI = (PtrGetCharWidthI)QSystemLibrary::resolve(QStringLiteral("gdi32"), "GetCharWidthI");
 }
 
-static inline quint32 getUInt(unsigned char *p)
-{
-    quint32 val;
-    val = *p++ << 24;
-    val |= *p++ << 16;
-    val |= *p++ << 8;
-    val |= *p;
-
-    return val;
-}
-
 static inline quint16 getUShort(unsigned char *p)
 {
     quint16 val;
@@ -170,6 +154,12 @@ bool QWindowsFontEngine::hasEbdtTable() const
     return GetFontData(hdc, MAKE_TAG('E', 'B', 'D', 'T'), 0, 0, 0) != GDI_ERROR;
 }
 
+static inline QString stringFromOutLineTextMetric(const OUTLINETEXTMETRIC *otm, PSTR offset)
+{
+    const uchar *p = reinterpret_cast<const uchar *>(otm) + quintptr(offset);
+    return QString::fromWCharArray(reinterpret_cast<const wchar_t *>(p));
+}
+
 void QWindowsFontEngine::getCMap()
 {
     ttf = (bool)(tm.tmPitchAndFamily & TMPF_TRUETYPE) || hasCMapTable();
@@ -193,11 +183,12 @@ void QWindowsFontEngine::getCMap()
     _faceId.index = 0;
     if(cmap) {
         OUTLINETEXTMETRIC *otm = getOutlineTextMetric(hdc);
-        designToDevice = QFixed((int)otm->otmEMSquare)/QFixed::fromReal(fontDef.pixelSize);
-        unitsPerEm = otm->otmEMSquare;
-        x_height = (int)otm->otmsXHeight;
-        loadKerningPairs(QFixed((int)otm->otmEMSquare)/int(otm->otmTextMetrics.tmHeight));
-        _faceId.filename = QFile::encodeName(QString::fromWCharArray((wchar_t *)((char *)otm + (quintptr)otm->otmpFullName)));
+        unitsPerEm = int(otm->otmEMSquare);
+        const QFixed unitsPerEmF(unitsPerEm);
+        designToDevice = unitsPerEmF / QFixed::fromReal(fontDef.pixelSize);
+        x_height = int(otm->otmsXHeight);
+        loadKerningPairs(unitsPerEmF / int(otm->otmTextMetrics.tmHeight));
+        _faceId.filename = QFile::encodeName(stringFromOutLineTextMetric(otm, otm->otmpFullName));
         lineWidth = otm->otmsUnderscoreSize;
         fsType = otm->otmfsType;
         free(otm);
@@ -231,17 +222,10 @@ int QWindowsFontEngine::getGlyphIndexes(const QChar *str, int numChars, QGlyphLa
             }
         } else {
 #endif
-            wchar_t first = tm.tmFirstChar;
-            wchar_t last = tm.tmLastChar;
-
             QStringIterator it(str, str + numChars);
             while (it.hasNext()) {
                 const uint uc = it.next();
-                if (
-#ifdef Q_DEAD_CODE_FROM_QT4_WINCE
-                    tm.tmFirstChar > 60000 ||
-#endif
-                         uc >= first && uc <= last)
+                if (uc >= tm.tmFirstChar && uc <= tm.tmLastChar)
                     glyphs->glyphs[glyph_pos] = uc;
                 else
                     glyphs->glyphs[glyph_pos] = 0;
@@ -272,7 +256,6 @@ QWindowsFontEngine::QWindowsFontEngine(const QString &name,
     m_logfont(lf),
     ttf(0),
     hasOutline(0),
-    lw(0),
     cmap(0),
     cmapSize(0),
     lbearing(SHRT_MIN),
@@ -351,11 +334,9 @@ glyph_t QWindowsFontEngine::glyphIndex(uint ucs4) const
             glyph = getTrueTypeGlyphIndex(cmap, cmapSize, ucs4 + 0xf000);
     } else if (ttf) {
         glyph = getTrueTypeGlyphIndex(cmap, cmapSize, ucs4);
-#else
-    if (tm.tmFirstChar > 60000) {
-        glyph = ucs4;
+    } else
 #endif
-    } else if (ucs4 >= tm.tmFirstChar && ucs4 <= tm.tmLastChar) {
+    if (ucs4 >= tm.tmFirstChar && ucs4 <= tm.tmLastChar) {
         glyph = ucs4;
     } else {
         glyph = 0;
@@ -408,9 +389,9 @@ void QWindowsFontEngine::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::Shape
         for(int i = 0; i < glyphs->numGlyphs; i++) {
             unsigned int glyph = glyphs->glyphs[i];
             if(int(glyph) >= designAdvancesSize) {
-                int newSize = (glyph + 256) >> 8 << 8;
-                designAdvances = q_check_ptr((QFixed *)realloc(designAdvances,
-                            newSize*sizeof(QFixed)));
+                const int newSize = int(glyph + 256) >> 8 << 8;
+                designAdvances = reinterpret_cast<QFixed *>(realloc(designAdvances, size_t(newSize) * sizeof(QFixed)));
+                Q_CHECK_PTR(designAdvances);
                 for(int i = designAdvancesSize; i < newSize; ++i)
                     designAdvances[i] = -1000000;
                 designAdvancesSize = newSize;
@@ -432,9 +413,9 @@ void QWindowsFontEngine::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::Shape
             unsigned int glyph = glyphs->glyphs[i];
 
             if (glyph >= widthCacheSize) {
-                int newSize = (glyph + 256) >> 8 << 8;
-                widthCache = q_check_ptr((unsigned char *)realloc(widthCache,
-                            newSize*sizeof(QFixed)));
+                const uint newSize = (glyph + 256) >> 8 << 8;
+                widthCache = reinterpret_cast<unsigned char *>(realloc(widthCache, newSize * sizeof(QFixed)));
+                Q_CHECK_PTR(widthCache);
                 memset(widthCache + widthCacheSize, 0, newSize - widthCacheSize);
                 widthCacheSize = newSize;
             }
@@ -454,7 +435,7 @@ void QWindowsFontEngine::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::Shape
                         ++chrLen;
                     }
                     SIZE size = {0, 0};
-                    GetTextExtentPoint32(hdc, (wchar_t *)ch, chrLen, &size);
+                    GetTextExtentPoint32(hdc, reinterpret_cast<const wchar_t *>(ch), chrLen, &size);
                     width = size.cx;
                 } else {
                     calculateTTFGlyphWidth(hdc, glyph, width);
@@ -462,7 +443,7 @@ void QWindowsFontEngine::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::Shape
                 glyphs->advances[i] = width;
                 // if glyph's within cache range, store it for later
                 if (width > 0 && width < 0x100)
-                    widthCache[glyph] = width;
+                    widthCache[glyph] = uchar(width);
             }
         }
 
@@ -503,10 +484,10 @@ bool QWindowsFontEngine::getOutlineMetrics(glyph_t glyph, const QTransform &t, g
         // results provided when transforming via MAT2 does not
         // match the glyphs that are drawn using a WorldTransform
         XFORM xform;
-        xform.eM11 = t.m11();
-        xform.eM12 = t.m12();
-        xform.eM21 = t.m21();
-        xform.eM22 = t.m22();
+        xform.eM11 = FLOAT(t.m11());
+        xform.eM12 = FLOAT(t.m12());
+        xform.eM21 = FLOAT(t.m21());
+        xform.eM22 = FLOAT(t.m22());
         xform.eDx = 0;
         xform.eDy = 0;
         SetGraphicsMode(hdc, GM_ADVANCED);
@@ -528,7 +509,8 @@ bool QWindowsFontEngine::getOutlineMetrics(glyph_t glyph, const QTransform &t, g
 
     if (res != GDI_ERROR) {
         *metrics = glyph_metrics_t(gm.gmptGlyphOrigin.x, -gm.gmptGlyphOrigin.y,
-                                  (int)gm.gmBlackBoxX, (int)gm.gmBlackBoxY, gm.gmCellIncX, gm.gmCellIncY);
+                                   int(gm.gmBlackBoxX), int(gm.gmBlackBoxY),
+                                   gm.gmCellIncX, gm.gmCellIncY);
         return true;
     } else {
         return false;
@@ -547,7 +529,7 @@ glyph_metrics_t QWindowsFontEngine::boundingBox(glyph_t glyph, const QTransform
 
     if (!ttf && !success) {
         // Bitmap fonts
-        wchar_t ch = glyph;
+        wchar_t ch = wchar_t(glyph);
         ABCFLOAT abc;
         GetCharABCWidthsFloat(hdc, ch, ch, &abc);
         int width = qRound(abc.abcfB);
@@ -812,7 +794,7 @@ static bool addGlyphToPath(glyph_t glyph, const QFixedPoint &position, HDC hdc,
             return false;
         // #### obey scale
         *metric = glyph_metrics_t(gMetric.gmptGlyphOrigin.x, -gMetric.gmptGlyphOrigin.y,
-                                  (int)gMetric.gmBlackBoxX, (int)gMetric.gmBlackBoxY,
+                                  int(gMetric.gmBlackBoxX), int(gMetric.gmBlackBoxY),
                                   gMetric.gmCellIncX, gMetric.gmCellIncY);
     }
 #endif
@@ -822,17 +804,15 @@ static bool addGlyphToPath(glyph_t glyph, const QFixedPoint &position, HDC hdc,
     if (ttf)
         glyphFormat |= GGO_GLYPH_INDEX;
 
-    int bufferSize = GDI_ERROR;
-    bufferSize = GetGlyphOutline(hdc, glyph, glyphFormat, &gMetric, 0, 0, &mat);
-    if ((DWORD)bufferSize == GDI_ERROR) {
+    const DWORD bufferSize = GetGlyphOutline(hdc, glyph, glyphFormat, &gMetric, 0, 0, &mat);
+    if (bufferSize == GDI_ERROR)
         return false;
-    }
 
-    void *dataBuffer = new char[bufferSize];
+    char *dataBuffer = new char[bufferSize];
     DWORD ret = GDI_ERROR;
     ret = GetGlyphOutline(hdc, glyph, glyphFormat, &gMetric, bufferSize, dataBuffer, &mat);
     if (ret == GDI_ERROR) {
-        delete [](char *)dataBuffer;
+        delete [] dataBuffer;
         return false;
     }
 
@@ -845,20 +825,18 @@ static bool addGlyphToPath(glyph_t glyph, const QFixedPoint &position, HDC hdc,
     }
 #endif
 
-    int offset = 0;
-    int headerOffset = 0;
-    TTPOLYGONHEADER *ttph = 0;
+    DWORD offset = 0;
+    DWORD headerOffset = 0;
 
     QPointF oset = position.toPointF();
     while (headerOffset < bufferSize) {
-        ttph = (TTPOLYGONHEADER*)((char *)dataBuffer + headerOffset);
+        const TTPOLYGONHEADER *ttph = reinterpret_cast<const TTPOLYGONHEADER *>(dataBuffer + headerOffset);
 
         QPointF lastPoint(qt_to_qpointf(ttph->pfxStart, scale));
         path->moveTo(lastPoint + oset);
         offset += sizeof(TTPOLYGONHEADER);
-        TTPOLYCURVE *curve;
-        while (offset<int(headerOffset + ttph->cb)) {
-            curve = (TTPOLYCURVE*)((char*)(dataBuffer) + offset);
+        while (offset < headerOffset + ttph->cb) {
+            const TTPOLYCURVE *curve = reinterpret_cast<const TTPOLYCURVE *>(dataBuffer + offset);
             switch (curve->wType) {
             case TT_PRIM_LINE: {
                 for (int i=0; i<curve->cpfx; ++i) {
@@ -903,7 +881,7 @@ static bool addGlyphToPath(glyph_t glyph, const QFixedPoint &position, HDC hdc,
         path->closeSubpath();
         headerOffset += ttph->cb;
     }
-    delete [] (char*)dataBuffer;
+    delete [] dataBuffer;
 
     return true;
 }
@@ -1001,15 +979,15 @@ QFontEngine::Properties QWindowsFontEngine::properties() const
     Properties p;
     p.emSquare = unitsPerEm;
     p.italicAngle = otm->otmItalicAngle;
-    p.postscriptName = QString::fromWCharArray((wchar_t *)((char *)otm + (quintptr)otm->otmpFamilyName)).toLatin1();
-    p.postscriptName += QString::fromWCharArray((wchar_t *)((char *)otm + (quintptr)otm->otmpStyleName)).toLatin1();
-    p.postscriptName = QFontEngine::convertToPostscriptFontFamilyName(p.postscriptName);
+    const QByteArray name = stringFromOutLineTextMetric(otm, otm->otmpFamilyName).toLatin1()
+        + stringFromOutLineTextMetric(otm, otm->otmpStyleName).toLatin1();
+    p.postscriptName = QFontEngine::convertToPostscriptFontFamilyName(name);
     p.boundingBox = QRectF(otm->otmrcFontBox.left, -otm->otmrcFontBox.top,
                            otm->otmrcFontBox.right - otm->otmrcFontBox.left,
                            otm->otmrcFontBox.top - otm->otmrcFontBox.bottom);
     p.ascent = otm->otmAscent;
     p.descent = -otm->otmDescent;
-    p.leading = (int)otm->otmLineGap;
+    p.leading = int(otm->otmLineGap);
     p.capHeight = 0;
     p.lineWidth = otm->otmsUnderscoreSize;
     free(otm);
@@ -1075,10 +1053,10 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
     XFORM xform;
 
     if (has_transformation) {
-        xform.eM11 = t.m11();
-        xform.eM12 = t.m12();
-        xform.eM21 = t.m21();
-        xform.eM22 = t.m22();
+        xform.eM11 = FLOAT(t.m11());
+        xform.eM12 = FLOAT(t.m12());
+        xform.eM21 = FLOAT(t.m21());
+        xform.eM22 = FLOAT(t.m22());
         xform.eDx = margin;
         xform.eDy = margin;
 
@@ -1088,7 +1066,7 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
         SetWorldTransform(hdc, &xform);
         HGDIOBJ old_font = SelectObject(hdc, font);
 
-        int ggo_options = GGO_METRICS | (ttf ? GGO_GLYPH_INDEX : 0);
+        const UINT ggo_options = GGO_METRICS | (ttf ? GGO_GLYPH_INDEX : 0);
         GLYPHMETRICS tgm;
         MAT2 mat;
         memset(&mat, 0, sizeof(mat));
@@ -1102,13 +1080,13 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
         SelectObject(hdc, old_font);
 
         if (result == GDI_ERROR) {
-            const int errorCode = GetLastError();
+            const int errorCode = int(GetLastError());
             qErrnoWarning(errorCode, "QWinFontEngine: unable to query transformed glyph metrics (GetGlyphOutline() failed, error %d)...", errorCode);
             return 0;
         }
 
-        iw = tgm.gmBlackBoxX;
-        ih = tgm.gmBlackBoxY;
+        iw = int(tgm.gmBlackBoxX);
+        ih = int(tgm.gmBlackBoxY);
 
         xform.eDx -= tgm.gmptGlyphOrigin.x;
         xform.eDy += tgm.gmptGlyphOrigin.y;
@@ -1145,11 +1123,11 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
     if (has_transformation) {
         SetGraphicsMode(hdc, GM_ADVANCED);
         SetWorldTransform(hdc, &xform);
-        ExtTextOut(hdc, 0, 0, options, 0, (LPCWSTR) &glyph, 1, 0);
+        ExtTextOut(hdc, 0, 0, options, 0, reinterpret_cast<LPCWSTR>(&glyph), 1, 0);
     } else
 #endif // !Q_OS_WINCE
     {
-        ExtTextOut(hdc, -gx + margin, -gy + margin, options, 0, (LPCWSTR) &glyph, 1, 0);
+        ExtTextOut(hdc, -gx + margin, -gy + margin, options, 0, reinterpret_cast<LPCWSTR>(&glyph), 1, 0);
     }
 
     SelectObject(hdc, old_font);
@@ -1170,7 +1148,9 @@ glyph_metrics_t QWindowsFontEngine::alphaMapBoundingBox(glyph_t glyph, QFixed, c
 QImage QWindowsFontEngine::alphaMapForGlyph(glyph_t glyph, const QTransform &xform)
 {
     HFONT font = hfont;
-    if (m_fontEngineData->clearTypeEnabled) {
+
+    bool clearTypeTemporarilyDisabled = (m_fontEngineData->clearTypeEnabled && m_logfont.lfQuality != NONANTIALIASED_QUALITY);
+    if (clearTypeTemporarilyDisabled) {
         LOGFONT lf = m_logfont;
         lf.lfQuality = ANTIALIASED_QUALITY;
         font = CreateFontIndirect(&lf);
@@ -1178,7 +1158,7 @@ QImage QWindowsFontEngine::alphaMapForGlyph(glyph_t glyph, const QTransform &xfo
     QImage::Format mask_format = QWindowsNativeImage::systemFormat();
     mask_format = QImage::Format_RGB32;
 
-    QWindowsNativeImage *mask = drawGDIGlyph(font, glyph, 0, xform, mask_format);
+    const QWindowsNativeImage *mask = drawGDIGlyph(font, glyph, 0, xform, mask_format);
     if (mask == 0) {
         if (m_fontEngineData->clearTypeEnabled)
             DeleteObject(font);
@@ -1193,11 +1173,11 @@ QImage QWindowsFontEngine::alphaMapForGlyph(glyph_t glyph, const QTransform &xfo
     for (int y=0; y<mask->height(); ++y) {
         uchar *dest = alphaMap.scanLine(y);
         if (mask->image().format() == QImage::Format_RGB16) {
-            const qint16 *src = (qint16 *) ((const QImage &) mask->image()).scanLine(y);
+            const qint16 *src = reinterpret_cast<const qint16 *>(mask->image().constScanLine(y));
             for (int x=0; x<mask->width(); ++x)
                 dest[x] = 255 - qGray(src[x]);
         } else {
-            const uint *src = (uint *) ((const QImage &) mask->image()).scanLine(y);
+            const uint *src = reinterpret_cast<const uint *>(mask->image().constScanLine(y));
             for (int x=0; x<mask->width(); ++x) {
                 if (QWindowsNativeImage::systemFormat() == QImage::Format_RGB16)
                     dest[x] = 255 - qGray(src[x]);
@@ -1209,7 +1189,7 @@ QImage QWindowsFontEngine::alphaMapForGlyph(glyph_t glyph, const QTransform &xfo
 
     // Cleanup...
     delete mask;
-    if (m_fontEngineData->clearTypeEnabled) {
+    if (clearTypeTemporarilyDisabled) {
         DeleteObject(font);
     }
 
@@ -1225,11 +1205,11 @@ QImage QWindowsFontEngine::alphaRGBMapForGlyph(glyph_t glyph, QFixed, const QTra
 
     UINT contrast;
     SystemParametersInfo(SPI_GETFONTSMOOTHINGCONTRAST, 0, &contrast, 0);
-    SystemParametersInfo(SPI_SETFONTSMOOTHINGCONTRAST, 0, (void *) 1000, 0);
+    SystemParametersInfo(SPI_SETFONTSMOOTHINGCONTRAST, 0, reinterpret_cast<void *>(quintptr(1000)), 0);
 
     int margin = glyphMargin(QFontEngine::Format_A32);
     QWindowsNativeImage *mask = drawGDIGlyph(font, glyph, margin, t, QImage::Format_RGB32);
-    SystemParametersInfo(SPI_SETFONTSMOOTHINGCONTRAST, 0, (void *) quintptr(contrast), 0);
+    SystemParametersInfo(SPI_SETFONTSMOOTHINGCONTRAST, 0, reinterpret_cast<void *>(quintptr(contrast)), 0);
 
     if (mask == 0)
         return QImage();
@@ -1242,7 +1222,7 @@ QImage QWindowsFontEngine::alphaRGBMapForGlyph(glyph_t glyph, QFixed, const QTra
     QImage rgbMask(mask->width(), mask->height(), QImage::Format_RGB32);
     for (int y=0; y<mask->height(); ++y) {
         uint *dest = (uint *) rgbMask.scanLine(y);
-        const uint *src = (uint *) source.scanLine(y);
+        const uint *src = reinterpret_cast<const uint *>(source.constScanLine(y));
         for (int x=0; x<mask->width(); ++x) {
             dest[x] = 0xffffffff - (0x00ffffff & src[x]);
         }
@@ -1276,6 +1256,11 @@ QFontEngine *QWindowsFontEngine::cloneWithSize(qreal pixelSize) const
     return fontEngine;
 }
 
+Qt::HANDLE QWindowsFontEngine::handle() const
+{
+    return hfont;
+}
+
 void QWindowsFontEngine::initFontInfo(const QFontDef &request,
                                       int dpi)
 {
@@ -1357,6 +1342,12 @@ QFontEngine *QWindowsMultiFontEngine::loadEngine(int at)
                 QWindowsFontEngineDirectWrite *fedw = new QWindowsFontEngineDirectWrite(directWriteFontFace,
                                                                                         fontEngine->fontDef.pixelSize,
                                                                                         data);
+                if (fontEngine->fontDef.weight > QFont::Normal)
+                    fedw->fontDef.weight = fontEngine->fontDef.weight;
+                if (fontEngine->fontDef.style > QFont::StyleNormal)
+                    fedw->fontDef.style = fontEngine->fontDef.style;
+                fedw->fontDef.family = fam;
+                fedw->fontDef.hintingPreference = fontEngine->fontDef.hintingPreference;
                 return fedw;
             } else {
                 qErrnoWarning("%s: CreateFontFace failed", __FUNCTION__);
@@ -1368,7 +1359,14 @@ QFontEngine *QWindowsMultiFontEngine::loadEngine(int at)
     // Get here if original font is not DirectWrite or DirectWrite creation failed for some
     // reason
 
-    return new QWindowsFontEngine(fam, lf, data);
+    QFontEngine *fe = new QWindowsFontEngine(fam, lf, data);
+    if (fontEngine->fontDef.weight > QFont::Normal)
+        fe->fontDef.weight = fontEngine->fontDef.weight;
+    if (fontEngine->fontDef.style > QFont::StyleNormal)
+        fe->fontDef.style = fontEngine->fontDef.style;
+    fe->fontDef.family = fam;
+    fe->fontDef.hintingPreference = fontEngine->fontDef.hintingPreference;
+    return fe;
 }
 
 bool QWindowsFontEngine::supportsTransformation(const QTransform &transform) const
