@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -81,6 +81,7 @@ private slots:
     void clone();
     void sortChildren();
     void subclassing();
+    void lessThan();
 };
 
 tst_QStandardItem::tst_QStandardItem()
@@ -277,9 +278,12 @@ void tst_QStandardItem::getSetFlags()
     QVERIFY(item.isCheckable());
     QCOMPARE(item.checkState(), Qt::Unchecked);
     QVERIFY(item.flags() & Qt::ItemIsUserCheckable);
-    item.setTristate(true);
-    QVERIFY(item.isTristate());
-    QVERIFY(item.flags() & Qt::ItemIsTristate);
+    item.setUserTristate(true);
+    QVERIFY(item.isUserTristate());
+    QVERIFY(item.flags() & Qt::ItemIsUserTristate);
+    item.setAutoTristate(true);
+    QVERIFY(item.isAutoTristate());
+    QVERIFY(item.flags() & Qt::ItemIsAutoTristate);
 #ifndef QT_NO_DRAGANDDROP
     item.setDragEnabled(true);
     QVERIFY(item.isDragEnabled());
@@ -305,10 +309,12 @@ void tst_QStandardItem::getSetFlags()
     item.setCheckable(false);
     QVERIFY(!item.isCheckable());
     QVERIFY(!(item.flags() & Qt::ItemIsUserCheckable));
-    QVERIFY(item.isTristate());
-    item.setTristate(false);
-    QVERIFY(!item.isTristate());
-    QVERIFY(!(item.flags() & Qt::ItemIsTristate));
+    item.setUserTristate(false);
+    QVERIFY(!item.isUserTristate());
+    QVERIFY(!(item.flags() & Qt::ItemIsUserTristate));
+    item.setAutoTristate(false);
+    QVERIFY(!item.isAutoTristate());
+    QVERIFY(!(item.flags() & Qt::ItemIsAutoTristate));
 #ifndef QT_NO_DRAGANDDROP
     QVERIFY(item.isDragEnabled());
     item.setDragEnabled(false);
@@ -324,6 +330,13 @@ void tst_QStandardItem::getSetFlags()
     item.setCheckState(Qt::Checked);
     item.setCheckable(true);
     QCOMPARE(item.checkState(), Qt::Checked);
+
+    // deprecated API
+    item.setTristate(true);
+    QVERIFY(item.isTristate());
+    QVERIFY(item.flags() & Qt::ItemIsTristate);
+    item.setTristate(false);
+    QVERIFY(!(item.flags() & Qt::ItemIsTristate));
 }
 
 void tst_QStandardItem::getSetRowAndColumnCount()
@@ -1095,5 +1108,19 @@ void tst_QStandardItem::subclassing()
     QCOMPARE(item->child(2), (QStandardItem*)child1);
 }
 
+void tst_QStandardItem::lessThan()
+{
+    QStandardItem stringA("A");
+    QStandardItem stringB("B");
+    QStandardItem invalid1;
+    QStandardItem invalid2;
+    QVERIFY(stringA < stringB);
+    QVERIFY(!(stringB < stringA));
+    // Items with invalid data go to the end.
+    QVERIFY(stringA < invalid1);
+    QVERIFY(!(invalid1 < stringA));
+    QVERIFY(!(invalid1 < invalid2));
+}
+
 QTEST_MAIN(tst_QStandardItem)
 #include "tst_qstandarditem.moc"
